/*
 * main.c
 *
 * Let LED0 and LED1 blink with different frequencies using multitasking. 
 *
 * Acquired skill: Understand how tasks are created and how they work.
 *
 * Please see www.freertos.org for more information.
 * vTaskDelay: FreeRTOS/API Reference/Task Control
 * xTaskCreate: FreeRTOS/API Reference/Task Creation
 * vTaskStartScheduler: FreeRTOS/API Reference/RTOS Kernel Control
 *
 *  Created: 2015-01-26
 * Modified: 2018-06-30
 *           2019-02-05
 *   Author: Wolfgang Neff
 */

#include <avr/io.h>
#include <FreeRTOS.h>
#include <task.h>

#include "board.h"

void vLED0Task(void* pvParameters)
{
	// No task initialization.
	while (1) {
		// Toggle LED0.
		LED_PORT.OUT ^= LED0_PIN_bm;
		// Wait a bit.
		vTaskDelay(pdMS_TO_TICKS(500));
	}
}

void vLED1Task(void* pvParameters)
{
	while (1) {
		LED_PORT.OUT ^= LED1_PIN_bm;
		vTaskDelay(pdMS_TO_TICKS(250));
	}
}

int main(void)
{
	// Initialize the hardware.
	LED_PORT.DIR = LED0_PIN_bm | LED1_PIN_bm;
	LED_PORT.OUT = LED0_PIN_bm | LED1_PIN_bm;
	
	// Create the tasks.
	xTaskCreate(vLED0Task, "LED0Task", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL);
	xTaskCreate(vLED1Task, "LED1Task", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL);
	
	// Start the scheduler.
	vTaskStartScheduler();
}
