/*
 * main.c
 *
 * A tasks waits until another task tells it that a button is pressed. Then it
 * lets LED0 light up. The LED task is dependent on the button task. The button
 * task synchronizes the LED task.
 *
 * Acquired skill: Understand how tasks are synchronized and how binary semaphores work.
 *
 * Please see www.freertos.org for more information.
 * xSemaphoreTake: FreeRTOS/API Reference/Semaphore / Mutexes
 * xSemaphoreGive: FreeRTOS/API Reference/Semaphore / Mutexes
 * xSemaphoreCreateBinary: FreeRTOS/API Reference/Semaphore / Mutexes
 *
 *  Created: 2015-01-27
 * Modified: 2018-06-30
 *   Author: Wolfgang Neff
 */

#include <avr/io.h>
#include <FreeRTOS.h>
#include <semphr.h>
#include <task.h>

#include "board.h"

SemaphoreHandle_t xSemaphore;

void vLEDTask(void* pvParameters)
{
	while (1) {
		xSemaphoreTake(xSemaphore, portMAX_DELAY);
		LED_PORT.OUT &= ~LED0_PIN_bm;
		vTaskDelay(pdMS_TO_TICKS(1000));
		LED_PORT.OUT |= LED0_PIN_bm;
	}
}

void vButtonTask(void* pvParameters)
{
	while (1) {
		if (!(BUTTON_LOW_PORT.IN & BUTTON0_PIN_bm)) {
			xSemaphoreGive(xSemaphore);
		}
		vTaskDelay(pdMS_TO_TICKS(100));
	}
}

int main(void)
{
	// Initialize the LED.
    LED_PORT.DIR = LED0_PIN_bm;
    LED_PORT.OUT = LED0_PIN_bm;

	// Initialize the push-button.
    BUTTON0_PINCTRL = PORT_OPC_PULLUP_gc;

    // Create the binary semaphore.
	xSemaphore = xSemaphoreCreateBinary();

	// Create the tasks.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+2, NULL);
	xTaskCreate(vButtonTask, "ButtonTask", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL);

	// Start the scheduler.
	vTaskStartScheduler();
}
